package sun.nio.fs;

import java.util.Locale;
import java.io.IOException;
import java.net.URLConnection;
import java.nio.file.Path;
import java.nio.file.spi.FileTypeDetector;

public abstract class AbstractFileTypeDetector extends FileTypeDetector
{
    private static final String TSPECIALS = "()<>@,;:/[]?=\\\"";
    
    protected AbstractFileTypeDetector() {
    }
    
    @Override
    public final String probeContentType(Path path) throws IOException {
        if (path == null) {
            throw new NullPointerException("'file' is null");
        }
        String s = this.implProbeContentType(path);
        if (s == null) {
            Path fileName = path.getFileName();
            if (fileName != null) {
                s = URLConnection.getFileNameMap().getContentTypeFor(fileName.toString());
            }
        }
        return (s == null) ? null : parse(s);
    }
    
    protected abstract String implProbeContentType(Path p0) throws IOException;
    
    private static String parse(String s) {
        int index = s.indexOf(47);
        int index2 = s.indexOf(59);
        if (index < 0) {
            return null;
        }
        String lowerCase = s.substring(0, index).trim().toLowerCase(Locale.ENGLISH);
        if (!isValidToken(lowerCase)) {
            return null;
        }
        String lowerCase2 = ((index2 < 0) ? s.substring(index + 1) : s.substring(index + 1, index2)).trim().toLowerCase(Locale.ENGLISH);
        if (!isValidToken(lowerCase2)) {
            return null;
        }
        StringBuilder sb = new StringBuilder(lowerCase.length() + lowerCase2.length() + 1);
        sb.append(lowerCase);
        sb.append('/');
        sb.append(lowerCase2);
        return sb.toString();
    }
    
    private static boolean isTokenChar(char ch) {
        return ch > ' ' && ch < '\u007f' && "()<>@,;:/[]?=\\\"".indexOf((int)ch) < 0;
    }
    
    private static boolean isValidToken(String s) {
        int length = s.length();
        if (length == 0) {
            return false;
        }
        int i = 0;
        while (i < length) {
            if (!isTokenChar(s.charAt(i))) {
                return false;
            }
            ++i;
        }
        return true;
    }
}
